% dsge_mhrun.m
% - script to run MH algorithm
% ------------------------------------------------------------------------
% Housekeeping
% ------------------------------------------------------------------------
% set clock
time0 = now;


% some option clearning
estinfo.nparticle = 0; %this program does not use PF -> force to zero
estinfo.ncore     = 1;
estinfo.parfor    = 0;
estinfo.mex       = 0;

% ------------------------------------------------------------------------
% Initialize Output Files
% ------------------------------------------------------------------------
% opath = [opath, '\mhrun', num2str(mhrun)];
chk_dir(opath);

% start log file
whattimeisit = clock;
logfilename = ['log_mh_run', num2str(mhrun), num2str(floor(whattimeisit(1,2:6))), '.log'];
cd(opath)
diary(logfilename)
cd(workpath)

% ------------------------------------------------------------------------
% Import prior specification
% ------------------------------------------------------------------------
% load prior
eval(['[prior, paraname] = prior_', specname,';'])

pshape   = prior(:,1);
pmean    = prior(:,2);
pstdd    = prior(:,3);
pmask    = prior(:,4);
pfix     = prior(:,5);
pmaskinv = 1- pmask;
pshape   = pshape.*pmaskinv;

% parameter specification: transformation
trspec      = prtr(modelname, paraname);
trspec(:,1) = trspec(:,1).*pmaskinv;
bounds      = trspec(:,2:3);

% ------------------------------------------------------------------------
% Import Posterior Max Results
% pmod, pmod_poast, pmod_pario, penalt, sigmult, pmod_std, recodeh
% ------------------------------------------------------------------------
if useHess == 1
    cd(lpath);
    % check
    if exist('hess.mat', 'file') == 0
        disp('Please run hessian program first !!!');
        disp('Stop this program');
        pause
    end
    load('hess.mat');
    cd(workpath);
elseif useHess == 2
    cd(lpath);
    % check
    if exist('hess_adaptive.mat', 'file') == 0
        disp('Please run hessian (adaptive) program first !!!');
        disp('Stop this program');
        pause
    end
    load('hess_adaptive.mat');
    cd(workpath);
else
    cd(opath);
    % check
    if exist('mh_pre.mat', 'file') == 0
        disp('Please run: pre-run MH program first !!!');
        disp('Stop this program');
        pause
    end
    load('mh_pre.mat');
    cd(workpath);
end
% set starting value as a posterior mode
para  = pmod;
npara = length(para);

% Loaded values
SIGPROPINV = hessian;          % negative inv hess
SIGSCALE   = sigmult;          % sqrt(negative inv hess)
SIGPROPdim = sum(pmaskinv);    % rank of hessian

% Log determinant for nonsingular part
% - log det of negative hess (not inv hess)
[u, s, v ] = svd(SIGPROPINV); % Inverse by Singular Value Decomposition (SVD)
SIGPROPlndet = 0;
for i=1:1:npara
    if i<=SIGPROPdim
        s(i,i) = 1/s(i,i);
        SIGPROPlndet = SIGPROPlndet + log(s(i,i));
    end
end

% Adaptation option
if npara > size(SIGSCALE)
    disp('NO RELEVANT HESSIAN INFO: ADAPTIVE ALGORITHM USED ...');
    estinfo.adapt = 1;
end

if estinfo.adapt == 1
    SIGSCALE = chol(0.1^2/sum(pmaskinv)*eye(size(para,1)))';
end

% configuration for random-block-mh
%estinfo.eachbin   = [6;6;6;6]; %number of parameters in each block
if size(estinfo.eachbin,2)>size(estinfo.eachbin,1)
    estinfo.eachbin = estinfo.eachbin';
end

estinfo.nblock    = size(estinfo.eachbin,1);
estinfo.blockbin  = zeros(estinfo.nblock,2);
is = 1; ie = [];
for ii=1:1:estinfo.nblock
    ie = is+estinfo.eachbin(ii)-1;
    estinfo.blockbin(ii,:) = [is, ie];
    is = ie+1;
end
if sum(estinfo.eachbin) ~= npara
    disp('--------------------------------------------------');
    disp('Wrong blockbin specificaiton!,.... Program stopped');
    disp(['# of para: ', num2str(npara)]);
    disp('--------------------------------------------------');
    pause
end

% ------------------------------------------------------------------------
% Print Out Loaded Files
% ------------------------------------------------------------------------
% print results
disp('-----------------------------------------');
disp(['Model          ', modelname]);
disp(['Prior          ', lprior]);
disp(['Dataset        ', ldata]);
disp(['Order          ', num2str(order_app)]);
disp(['# of particles ', num2str(n_particle)]);
disp('------------------------------------------');
disp('Esimation Results');
disp('------------------------------------------');
disp([{'parameters', 'mode', 'S.D.'}; [paraname num2cell(fixdec(pmod,3)) num2cell(fixdec(pmod_std,4))]]);
disp('------------------------------------------');
% log probabilities
tic;
[lnpost0, lnpy0, lnprior0, error, Xhat, Yhat, sol] = objfcn_mh(para, prior, bounds, estinfo, YY, modelname, order_app, n_particle, T0_);
[ny, nx]  = size(sol.ZZ);
time_once = toc;
disp('---------------------------------------------------');
disp(['Log posterior  at mode  :', num2str(lnpost0)]);
disp(['Log likelihood at mode  :', num2str(lnpy0)]);
disp(['Log prior      at mode  :', num2str(lnprior0)]);
disp(['Initial Posterior Evaluation took ', num2str(time_once), ' second']);
disp('---------------------------------------------------');

% ------------------------------------------------------------------------
% MH algorithm
% ------------------------------------------------------------------------
% define function handle for mh-algorithm
objfcn_mh_handle = @(x)objfcn_mh(x, prior, bounds, estinfo, YY, modelname, order_app, n_particle, T0_);

% dimensions
nobs  = size(YY,1);
nYY   = size(YY,2);

% Trembles the posterior mode to get Starting Value (para = pmod)
disp('Finding Starting Points ... ');
valid = 0;

while valid == 0
    
    paraold  = para + cc0*(SIGSCALE*randn(npara,1));
    paraold  = paraold.*pmaskinv + pfix.*pmask;
    [postold, likeold, priorold, error, Xhatold, Yhatold, solold, fulllikold] = objfcn_mh_handle(paraold);
    
    if isempty(Xhatold)
        Xhatlastold = zeros(1,nx);
    else
        Xhatlastold = Xhatold(end,:);
    end
    
    propdens = -0.5*SIGPROPdim*log(2*pi) -0.5*SIGPROPlndet ...
        -0.5*SIGPROPdim*log(cc0^2) ...
        -0.5*(paraold - para)'*SIGPROPINV*(paraold - para)/cc0^2;
    
    % stopping rule
    if postold > -10000
        valid = 1;
    end
    
end
disp('Starting MH-Algorithm Draws ...');
% matrices for storage
mh_rej      = zeros(nsimul,npara);

mh_like     = zeros(nsimul,1);
mh_post     = zeros(nsimul,1);
mh_error    = zeros(nsimul,1);
mh_para     = zeros(nsimul, npara);
mh_prop     = zeros(nsimul, 1);
mh_sol      = cell(nsimul,1);
% mh_Xhatlast = zeros(nsimul,size(Xhatlastold,2));
%mh_fulllik  = cell(nsimul,1);


counter = 0;
mh_time = tic;
for mh_indx = 1:1:nsimul
    
    % Adpatation
    if estinfo.adapt == 1
        % Adaptive proposal - Tuning parameter for adaptive proposal
        mhtune.d  = sum(pmaskinv);
        mhtune.j0 = mhtune.d*2; % first samples before adaptation
        mhtune.w1 = 0.05;
        mhtune.w2 = 1-mhtune.w1;
        mhtune.k1 = 0.1^2/mhtune.d;
        mhtune.k2 = 2.38^2/mhtune.d;
        mhtune.sqrtcovini = 0.1*SIGSCALE;
        
        if mh_indx < mhtune.j0
            
            %adapt_cov = mhtune.sqrtcovini*mhtune.sqrtcovini';
            adapt_cov  = mhtune.k1*eye(mhtune.d);
            adapt_cov = chol(adapt_cov)';
            adapt_cov = trans_fix(adapt_cov,prior);
        elseif mh_indx < estinfo.nadapt
            
            % empirical cov
            adapt_cov = cov(mh_para(1:mh_indx-1,logical(pmaskinv)));
            adapt_cov = mhtune.w1^2*mhtune.k1*eye(mhtune.d) + mhtune.w2^2*mhtune.k2*adapt_cov;
            adapt_cov = chol(adapt_cov)';
            adapt_cov = trans_fix(adapt_cov,prior);
            
        else
            adapt_cov = adapt_cov;
        end
    else
        adapt_cov = estinfo.cc*SIGSCALE;
    end
    
    % Propose new parameters
    rejold = nan(npara,1);
    
    [~,ix] = sort(rand(npara,1)); % random block index
    for ib = 1:1:estinfo.nblock; %iterate over #of blocks
        % [1] propose new block value at ib
        paranew = paraold; %to prepare propose
        selb    = ix(estinfo.blockbin(ib,1): estinfo.blockbin(ib,2)); %block selector
        paranew(selb,1) = paraold(selb,1) + adapt_cov(selb,selb)*randn(size(selb,1),1); %propose
        paranew = paranew.*pmaskinv + pfix.*pmask;
        
        
        % Evaluate log-posterior
        [postnew, likenew, priornew, error, Xhatnew, Yhatnew, solnew, fullliknew] = objfcn_mh_handle(paranew);
        
        propdens = -0.5*SIGPROPdim*log(2*pi) - 0.5*SIGPROPlndet ...
            -0.5*SIGPROPdim*log(estinfo.cc^2) ...
            -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/estinfo.cc^2;
        
        % Accept/reject decision
        mhR = min(1, exp(postnew - postold));
        
        if (rand < mhR) && (error == 0) % accept
            
            % update
            likeold = likenew;
            postold = postnew;
            paraold = paranew;
            solold  = solnew;
            Xhatlastold = Xhatnew(end,:);
            fulllikold  = fullliknew;
            
            rejold(selb,1) = 0;
            
        else % reject
            rejold(selb,1) = 1;
            
        end
    end % index for random blocking
    
    % save draws in output matrices
    mh_para(mh_indx,:)  = paraold';
    mh_like(mh_indx,1)  = likeold;
    mh_post(mh_indx,1)  = postold;
    mh_prop(mh_indx,1)  = propdens;
    mh_error(mh_indx,1) = error;
    mh_sol{mh_indx,1}   = solold;
    %mh_Xhatlast(mh_indx,:) = Xhatlastold;
    %mh_fulllik{mh_indx,1} = fulllikold;
    
    mh_rej(mh_indx,:) = rejold;
    
    % for every ncounter, we report and save
    counter = counter + 1;
    if counter == ncounter
        
        temp_mean  = fixdec(mean(mh_para(1:mh_indx,:)), 3);
        temp_std   = fixdec(std(mh_para(1:mh_indx,:)), 4);
        temp_rej   = fixdec(mean(mh_rej(1:mh_indx,:)),2);
        %temp_print = [{''; 'mean'; 'std'} [paraname'; num2cell(temp_mean); num2cell(temp_std)]];
        temp_print = [{''; 'mean'; 'std'; 'rej'} [paraname'; num2cell(temp_mean); num2cell(temp_std); num2cell(temp_rej)]];
        
        temp_time  = toc(mh_time);
        temp_time_once = temp_time/mh_indx;
        temp_time_remain = (nsimul - mh_indx)*temp_time_once;
        
        disp('------------   MH Run ----------------------');
        disp([num2str(mh_indx), ' th simulation is done.']);
        %disp(['rej rate      : ', num2str(mean(mh_rej(1:mh_indx,1)))]);
        disp(['rej rate      : ', num2str(mean(mean(mh_rej(1:mh_indx,:))))]);
        disp('--------------------------------------------');
        disp(['elapsed time  (sec) : '  num2str(temp_time)]);
        disp(['average time  (sec) : ' num2str(temp_time_once)]);
        disp(['time remained (sec) : ' num2str(temp_time_remain)]);
        disp('--------------------------------------------');
        disp(temp_print');
        disp('--------------------------------------------');
        disp('Error');
        disp('--------------------------------------------');
        tabulate(mh_error(1:mh_indx,:));
        disp('--------------------------------------------');
        % reset counter
        counter = 0;
    end
    
end


% calculate elapsed time and print into log
time1 = now;
disp(' ');
disp(' ');
disp('-------------------------------------------------------');
disp(['start       :  ' datestr(time0)]);
disp(['end         :  ' datestr(time1)]);
disp(['elapsed time:  ' datestr(time1-time0,13)]);
disp('-------------------------------------------------------');
disp(' ');


% ------------------------------------------------------------------------
% Save outputs
% ------------------------------------------------------------------------
savefilename = ['mh', num2str(mhrun), '.mat'];
cd(opath);
save(savefilename, 'mh_rej', 'mh_like', 'mh_post', ...
    'mh_error', 'mh_para', 'mh_prop', ...
    'mh_sol','estinfo');
%savefilename2 = ['mh', num2str(mhrun),'_fulllik', '.mat'];
%save(savefilename2, 'mh_fulllik');
cd(workpath);

% close log file
diary off



