% dsge_pred.m
%  - The program computes trajectories from the DSGE model.
%  - Direct translation of professor Schorfheide's gauss program
% INPUT: nuse_, npaths_, nv_, hmaxact, tiplus, YYplus
% 

% Load file
cd(lpath);
load(['mh', mhrun, '.mat']);
cd(workpath);

% ------------------------------------------------------------------------
% use every nuse-th draw
% ------------------------------------------------------------------------
% parasim     = forevery(pa(nburn+1:end,:),nuse);
solsim      = forevery(mh_sol(nburn+1:end,:),nuse_);
% Xhatlastsim = forevery(mh_Xhatlast(nburn+1:end,:),nuse);


% size info
nread = size(solsim,1);

% ------------------------------------------------------------------------
% generate draws from predictive density
% ------------------------------------------------------------------------
predsim = zeros(nread*npaths_, nv_*hmaxact);
predsimpt = zeros(nread, nv_*hmaxact);

for j=1:1:nread

    predsim((j-1)*npaths_+1:j*npaths_,:) ...
        = dsgepredmult_new(solsim{j,:}, solsim{j}.Xlastdraw', ...
                           hmaxact, npaths_, order_app);
                       
    predsimpt((j-1)*1+1:j*1,:) ...
        = dsgepredmult_pt_new(solsim{j,:}, solsim{j}.Xlastfix', ...
                           hmaxact, 1, order_app);
end

% save predictions
cd(opath);
save('predvar.mat', 'predsim', 'predsimpt');
cd(workpath);

% write out mean forecast

% start log file
whattimeisit = clock;
logfilename  = ['log_pred', num2str(floor(whattimeisit(1,2:6))), '.log'];
cd(opath);
diary(logfilename);
cd(workpath);

% disp results
disp('Predicted ( Average over trajectories) ');
[tiplus reshape(mean(predsim), hmaxact, nv_)]
disp('Actual');
[tiplus YYplus]

diary off
